@@ -14,47 +14,40 @@ MIPModel <- function() {
   )
 }
 
-#' @rdname linear-functions
-setClass("OmprLinearVariableCollection",
-  slots = c(map = "ANY")
-)
 
-#' @importFrom rlang abort
-#' @rdname linear-functions
-#' @param x some value
-#' @param i an index
-#' @param j an index
-#' @param ... more indexes
-#' @param drop unused
-setMethod(
-  "[", signature("OmprLinearVariableCollection",
-    i = "ANY", j = "ANY", drop = "missing"
-  ),
-  function(x, i, j, ..., drop) {
-    indexes <- c(
-      if (!missing(i)) i,
-      if (!missing(j)) j,
-      ...
-    )
-    var_name <- hash_var_indexes(indexes)
-    var <- x@map$get(var_name)
-    if (is.null(var)) {
-      abort("Variable not found for indexes")
-    }
-    var
+#' @export
+`[.OmprLinearVariableCollection` <- function(x, i, j, ..., drop) {
+  indexes <- c(
+    if (!missing(i)) i,
+    if (!missing(j)) j,
+    ...
+  )
+  var_name <- hash_var_indexes(indexes)
+  var <- x$map$get(var_name)
+  if (is.null(var)) {
+    abort("Variable not found for indexes")
   }
-)
+  var
+}
 
-#' @rdname linear-functions
-setMethod("length", signature("OmprLinearVariableCollection"), function(x) {
-  x@map$size()
-})
+n_columns <- function(x) {
+  UseMethod("n_columns")
+}
 
-#' @include linear-optimization-model-linear-functions.R
-#' @rdname linear-functions
-setMethod("length", signature("OmprLinearVariable"), function(x) {
+#' @export
+n_columns.OmprLinearVariableCollection <- function(x) {
+  x$map$size()
+}
+
+#' @export
+n_columns.OmprLinearVariable <- function(x) {
   1
-})
+}
+
+#' @export
+n_columns.LinearTerm <- function(x) {
+  1
+}
 
 #' @export
 #' @inheritParams print
@@ -134,7 +127,6 @@ add_variable.linear_optimization_model <- function(.model, .variable, ...,
   }
 
   current_column_count <- model$column_count
-  # TODO: batch grow vectors, e.g. lower/upper bounds
   if (is_indexed_var) {
     stopifnot(
       !is.null(variable_index_names),
@@ -145,22 +137,28 @@ add_variable.linear_optimization_model <- function(.model, .variable, ...,
       .env = parent.frame()
     )
     el_names <- lapply(vars, hash_var_indexes)
+    model$variable_bounds_lower <- c(
+      model$variable_bounds_lower,
+      rep.int(lb, length(vars))
+    )
+    model$variable_bounds_upper <- c(
+      model$variable_bounds_upper,
+      rep.int(ub, length(vars))
+    )
     vars <- lapply(vars, function(x) {
       current_column_count <<- current_column_count + 1
-      model$variable_bounds_lower[[current_column_count]] <<- lb
-      model$variable_bounds_upper[[current_column_count]] <<- ub
-      var <- new("OmprLinearVariable", column_idx = current_column_count)
+      var <- new_linear_variable(column_idx = current_column_count)
       new_linear_term(var, 1)
     })
     vars <- setNames(vars, el_names)
     map <- fastmap()
     map$mset(.list = vars)
-    new_variable <- new("OmprLinearVariableCollection", map = map)
+    new_variable <- new_variable_collection(map = map)
   } else {
     current_column_count <- current_column_count + 1
     model$variable_bounds_lower[[current_column_count]] <- lb
     model$variable_bounds_upper[[current_column_count]] <- ub
-    new_variable <- new("OmprLinearVariable", column_idx = current_column_count)
+    new_variable <- new_linear_variable(column_idx = current_column_count)
     new_variable <- new_linear_term(new_variable, 1)
   }
   model$column_count <- current_column_count
@@ -249,9 +247,9 @@ set_bounds_ineq <- function(.model, .expr, dots, env) {
   }
   for (constraint in constraints) {
     bound <- extract_bound_from_constraint(
-      constraint@lhs, constraint@rhs, constraint@sense
+      constraint$lhs, constraint$rhs, constraint$sense
     )
-    col_idx <- bound$variable@column_idx
+    col_idx <- bound$variable$column_idx
     numeric_bound <- bound$bound
     if (inherits(bound$sense, "LinearConstraintSenseLeq")) {
       .model$variable_bounds_upper[[col_idx]] <- numeric_bound
@@ -267,48 +265,33 @@ set_bounds_ineq <- function(.model, .expr, dots, env) {
   .model
 }
 
-#' Extract a scalar bound for a variable from a constraint
-#' @rdname extract_bound_from_constraint
-#' @include linear-optimization-model-linear-constraints.R
-#' @keywords internal
-setGeneric("extract_bound_from_constraint", function(lhs, rhs, sense) {
-  stop("not implemented") #nocovr
-})
+extract_bound_from_constraint <- function(lhs, rhs, sense) {
+  UseMethod("extract_bound_from_constraint", lhs)
+}
 
-#' @rdname extract_bound_from_constraint
-setMethod(
-  "extract_bound_from_constraint",
-  signature(
-    lhs = "LinearFunction", rhs = "numeric",
-    sense = "LinearConstraintSense"
-  ),
-  function(lhs, rhs, sense) {
-    stopifnot(length(lhs@terms) == 1,
-              lhs@constant == 0)
-    extract_bound_from_constraint(lhs@terms[[1]], rhs, sense)
-  }
-)
+#' @export
+extract_bound_from_constraint.LinearFunction <- function(lhs, rhs, sense) {
+  terms <- terms_list(lhs)
+  stopifnot(
+    length(terms) == 1,
+    lhs$constant == 0
+  )
+  extract_bound_from_constraint(terms[[1]], rhs, sense)
+}
 
-#' @rdname extract_bound_from_constraint
-setMethod(
-  "extract_bound_from_constraint",
-  signature(
-    lhs = "LinearTerm", rhs = "numeric",
-    sense = "LinearConstraintSense"
-  ),
-  function(lhs, rhs, sense) {
-    term <- lhs
-    coefficient <- term@coefficient
-    if (coefficient < 0) {
-      sense <- flip_constaint_sense(sense)
-    }
-    list(
-      sense = sense,
-      variable = term@variable,
-      bound = rhs / coefficient
-    )
+#' @export
+extract_bound_from_constraint.LinearTerm <- function(lhs, rhs, sense) {
+  term <- lhs
+  coefficient <- term$coefficient
+  if (coefficient < 0) {
+    sense <- flip_constaint_sense(sense)
   }
-)
+  list(
+    sense = sense,
+    variable = term$variable,
+    bound = rhs / coefficient
+  )
+}
 
 #' @importFrom listcomp gen_list
 set_bounds_old <- function(.model, expr, dots,
@@ -325,9 +308,9 @@ set_bounds_old <- function(.model, expr, dots,
   vars <- gen_list(!!expr, !!!dots, .env = eval_env)
   for (var in vars) {
     if (inherits(var, "LinearTerm")) {
-      var <- var@variable
+      var <- var$variable
     }
-    col_idx <- var@column_idx
+    col_idx <- var$column_idx
     if (!is.null(lb)) {
       .model$variable_bounds_lower[[col_idx]] <- lb
     }
@@ -405,7 +388,11 @@ sum_over <- function(.expr, ...) {
   vals <- gen_list(!!enquo(.expr), !!!enquos(...),
     .env = parent.frame()
   )
-  Reduce(`+`, vals, init = 0)
+  res <- 0
+  for (x in vals) {
+    res <- res + x
+  }
+  res
 }
 
 #' @export
@@ -428,12 +415,16 @@ add_constraint.linear_optimization_model <- function(.model, .constraint_expr,
   any_type_errors <- !all(vapply(
     constraints,
     function(x) {
-      inherits(x, "LinearConstraint")
+      inherits(x, "LinearConstraint") ||
+        (is.logical(x) && x)
     }, logical(1L)
   ))
   if (any_type_errors) {
-    abort("Some constraints are not proper linear constraints")
+    abort("Some constraints are not proper linear constraints or are not true.")
   }
+  constraints <- Filter(function(x) {
+    inherits(x, "LinearConstraint")
+  }, constraints)
   .model$constraints <- c(.model$constraints, constraints) # O(n)
   .model
 }
@@ -484,7 +475,7 @@ nvars.linear_optimization_model <- function(model) {
     binary = 0
   )
   Reduce(function(buckets, var_name) {
-    var_count <- length(model$variables[[var_name]])
+    var_count <- n_columns(model$variables[[var_name]])
     var_type <- model$variable_types[[var_name]]
     buckets[[var_type]] <- buckets[[var_type]] + var_count
     buckets
@@ -505,9 +496,9 @@ variable_types.linear_optimization_model <- function(model) {
     type <- model$variable_types[[var_name]]
     variable <- model$variables[[var_name]]
     column_indexes <- if (inherits(variable, "LinearTerm")) {
-      variable@variable@column_idx
+      variable$variable$column_idx
     } else {
-      vapply(variable@map$as_list(), function(x) x@variable@column_idx, numeric(1))
+      vapply(variable$map$as_list(), function(x) x$variable$column_idx, numeric(1))
     }
     data.frame(column_idx = column_indexes, type = type)
   })
@@ -531,13 +522,13 @@ variable_keys.linear_optimization_model <- function(model) {
     variable <- model$variables[[var_name]]
 
     column_indexes <- if (inherits(variable, "LinearTerm")) {
-      setNames(variable@variable@column_idx, var_name)
+      setNames(variable$variable$column_idx, var_name)
     } else {
-      var_names <- paste0(var_name, "[", variable@map$keys(), "]")
+      var_names <- paste0(var_name, "[", variable$map$keys(), "]")
       setNames(
         vapply(
-          variable@map$as_list(),
-          function(x) x@variable@column_idx,
+          variable$map$as_list(),
+          function(x) x$variable$column_idx,
           numeric(1)
         ),
         var_names
@@ -573,15 +564,15 @@ objective_function.linear_optimization_model <- function(model) {
     obj_constant <- if (inherits(obj_fun, "LinearTerm")) {
       0
     } else {
-      obj_fun@constant
+      obj_fun$constant
     }
     if (inherits(obj_fun, "LinearTerm")) {
-      obj[obj_fun@variable@column_idx] <- obj_fun@coefficient
+      obj[obj_fun$variable$column_idx] <- obj_fun$coefficient
     } else {
-      terms <- reduce_linear_function(obj_fun)@terms
+      terms <- terms_list(obj_fun)
       for (term in terms) {
-        if (term@coefficient != 0) {
-          obj[term@variable@column_idx] <- term@coefficient
+        if (term$coefficient != 0) {
+          obj[term$variable$column_idx] <- term$coefficient
         }
       }
     }
@@ -589,28 +580,6 @@ objective_function.linear_optimization_model <- function(model) {
   list(solution = obj, constant = obj_constant)
 }
 
-reduce_linear_function <- function(linear_function) {
-  stopifnot(inherits(linear_function, "LinearFunction"))
-  # a linear function can have multiple terms pointing to the same variable
-  # we need reduce, such that each variables only occurs once
-  term_map <- fastmap()
-  for (term in linear_function@terms) {
-    var_idx <- as.character(term@variable@column_idx)
-    old_term <- term_map$get(var_idx, NULL)
-    term_map$set(var_idx,
-      if (is.null(old_term)) {
-        term
-      } else {
-        new_linear_term(
-          old_term@variable, old_term@coefficient + term@coefficient
-        )
-      }
-    )
-  }
-  linear_function@terms <- setNames(term_map$as_list(), NULL)
-  linear_function
-}
-
 #' @export
 #' @importFrom Matrix sparseMatrix
 extract_constraints.linear_optimization_model <- function(model) {
@@ -620,17 +589,17 @@ extract_constraints.linear_optimization_model <- function(model) {
     return(list(
       matrix = sparseMatrix(i = numeric(), j = numeric(), dims = c(0, n_vars)),
       sense = character(0L),
-      rhs = numeric(0L))
-    )
+      rhs = numeric(0L)
+    ))
   }
   row_counter <- 0
   matrix_coefs <- lapply(model$constraints, function(constraint) {
-    lhs <- (constraint@lhs - constraint@rhs) + 0
+    lhs <- (constraint$lhs - constraint$rhs) + 0
     stopifnot(inherits(lhs, "LinearFunction"))
-    rhs_numeric <- -1 * lhs@constant
-    lhs_terms <- reduce_linear_function(lhs)@terms
-    coefs <- vapply(lhs_terms, function(x) x@coefficient, numeric(1))
-    cols <- vapply(lhs_terms, function(x) x@variable@column_idx, numeric(1))
+    rhs_numeric <- -1 * lhs$constant
+    lhs_terms <- terms_list(lhs)
+    coefs <- vapply(lhs_terms, function(x) x$coefficient, numeric(1))
+    cols <- vapply(lhs_terms, function(x) x$variable$column_idx, numeric(1))
     cols <- c(cols, n_vars + 1) # O(n)
     coefs <- c(coefs, rhs_numeric) # O(n)
     row_counter <<- row_counter + 1
@@ -647,7 +616,7 @@ extract_constraints.linear_optimization_model <- function(model) {
     x = values,
     dims = c(n_constraints, n_vars + 1)
   )
-  sense <- vapply(model$constraints, function(x) x@sense@sense, character(1))
+  sense <- vapply(model$constraints, function(x) x$sense$sense, character(1))
   list(
     matrix = matrix[, seq_len(ncol(matrix) - 1), drop = FALSE],
     sense = sense,
