@@ -3,11 +3,13 @@ from typing import List, Dict, Callable, Any
 from ompr.runner import RunningWorker, OMPRunner
 
 
-# base function to process tasks using OMPR
+# base (blocking) function to process tasks using OMPR
 def simple_process(
         tasks: List[Dict],      # tasks to process
         function: Callable,     # processing function
         num_workers: int=   4,
+        logger=             None,
+        loglevel=           30,
         **kwargs,
 ) -> List[Any]:
 
@@ -18,6 +20,8 @@ def simple_process(
     ompr = OMPRunner(
         rw_class=   SimpleRW,
         devices=    [None]*num_workers,
+        logger=     logger,
+        loglevel=   loglevel,
         **kwargs)
 
     ompr.process(tasks)
